/**
 *    Copyright (C) 2018-present MongoDB, Inc.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the Server Side Public License, version 1,
 *    as published by MongoDB, Inc.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    Server Side Public License for more details.
 *
 *    You should have received a copy of the Server Side Public License
 *    along with this program. If not, see
 *    <http://www.mongodb.com/licensing/server-side-public-license>.
 *
 *    As a special exception, the copyright holders give permission to link the
 *    code of portions of this program with the OpenSSL library under certain
 *    conditions as described in each individual source file and distribute
 *    linked combinations including the program with the OpenSSL library. You
 *    must comply with the Server Side Public License in all respects for
 *    all of the code used other than as permitted herein. If you modify file(s)
 *    with this exception, you may extend this exception to your version of the
 *    file(s), but you are not obligated to do so. If you do not wish to do so,
 *    delete this exception statement from your version. If you delete this
 *    exception statement from all source files in the program, then also delete
 *    it in the license file.
 */

#pragma once

#include "mongo/logv2/log_severity.h"
#include "mongo/rpc/message.h"
#include "mongo/util/future.h"

namespace mongo {
class OperationContext;
struct DbResponse;

/**
 * This is the entrypoint from the transport layer into mongod or mongos.
 */
class ServiceEntryPoint {
private:
    ServiceEntryPoint(const ServiceEntryPoint&) = delete;
    ServiceEntryPoint& operator=(const ServiceEntryPoint&) = delete;

protected:
    ServiceEntryPoint() = default;

public:
    virtual ~ServiceEntryPoint() = default;

    /**
     * Returns the current severity to log slow SessionWorkflow lifecycles. Returns a suppressed
     * severity every X seconds (defined in SEP::impl) to prevent logs from overwhelming the rest
     * of the system.
     */
    virtual logv2::LogSeverity slowSessionWorkflowLogSeverity() = 0;

    /**
     * Processes a request and fills out a DbResponse.
     */
    virtual Future<DbResponse> handleRequest(OperationContext* opCtx,
                                             const Message& request) noexcept = 0;
};

}  // namespace mongo
